@@ -11,7 +11,7 @@
 
 
 
-class op_cumsum_mat
+class op_cumsum
   {
   public:
   
@@ -19,20 +19,19 @@ class op_cumsum_mat
   inline static void apply_noalias(Mat<eT>& out, const Mat<eT>& X, const uword dim);
   
   template<typename T1>
-  inline static void apply(Mat<typename T1::elem_type>& out, const Op<T1,op_cumsum_mat>& in);
+  inline static void apply(Mat<typename T1::elem_type>& out, const Op<T1,op_cumsum>& in);
   };
 
 
 
-class op_cumsum_vec
+class op_cumsum_simple
   {
   public:
   
-  template<typename eT>
-  inline static void apply_noalias(Mat<eT>& out, const Mat<eT>& X);
-  
   template<typename T1>
-  inline static void apply(Mat<typename T1::elem_type>& out, const Op<T1,op_cumsum_vec>& in);
+  inline static void apply(Mat<typename T1::elem_type>& out, const Op<T1,op_cumsum_simple>& in);
   };
 
+
+
 //! @}
