@@ -1,5 +1,5 @@
-// Copyright (C) 2009-2011 Conrad Sanderson
-// Copyright (C) 2009-2011 NICTA (www.nicta.com.au)
+// Copyright (C) 2009-2015 Conrad Sanderson
+// Copyright (C) 2009-2015 NICTA (www.nicta.com.au)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -30,46 +30,48 @@ op_stddev::apply(Mat<typename T1::pod_type>& out, const mtOp<typename T1::pod_ty
   const uword norm_type = in.aux_uword_a;
   const uword dim       = in.aux_uword_b;
   
-  arma_debug_check( (norm_type > 1), "stddev(): incorrect usage. norm_type must be 0 or 1");
-  arma_debug_check( (dim > 1),       "stddev(): incorrect usage. dim must be 0 or 1"      );
+  arma_debug_check( (norm_type > 1), "stddev(): parameter 'norm_type' must be 0 or 1" );
+  arma_debug_check( (dim > 1),       "stddev(): parameter 'dim' must be 0 or 1"       );
   
   const uword X_n_rows = X.n_rows;
   const uword X_n_cols = X.n_cols;
   
   if(dim == 0)
     {
-    arma_extra_debug_print("op_stddev::apply(), dim = 0");
-
-    arma_debug_check( (X_n_rows == 0), "stddev(): given object has zero rows" );
+    arma_extra_debug_print("op_stddev::apply(): dim = 0");
     
-    out.set_size(1, X_n_cols);
+    out.set_size((X_n_rows > 0) ? 1 : 0, X_n_cols);
     
-    out_eT* out_mem = out.memptr();
-    
-    for(uword col=0; col<X_n_cols; ++col)
+    if(X_n_rows > 0)
       {
-      out_mem[col] = std::sqrt( op_var::direct_var( X.colptr(col), X_n_rows, norm_type ) );
+      out_eT* out_mem = out.memptr();
+      
+      for(uword col=0; col<X_n_cols; ++col)
+        {
+        out_mem[col] = std::sqrt( op_var::direct_var( X.colptr(col), X_n_rows, norm_type ) );
+        }
       }
     }
   else
   if(dim == 1)
     {
-    arma_extra_debug_print("op_stddev::apply(), dim = 1");
+    arma_extra_debug_print("op_stddev::apply(): dim = 1");
     
-    arma_debug_check( (X_n_cols == 0), "stddev(): given object has zero columns" );
-
-    out.set_size(X_n_rows, 1);
+    out.set_size(X_n_rows, (X_n_cols > 0) ? 1 : 0);
     
-    podarray<in_eT> dat(X_n_cols);
-    
-    in_eT*  dat_mem = dat.memptr();
-    out_eT* out_mem = out.memptr();
-    
-    for(uword row=0; row<X_n_rows; ++row)
+    if(X_n_cols > 0)
       {
-      dat.copy_row(X, row);
+      podarray<in_eT> dat(X_n_cols);
       
-      out_mem[row] = std::sqrt( op_var::direct_var( dat_mem, X_n_cols, norm_type) );
+      in_eT*  dat_mem = dat.memptr();
+      out_eT* out_mem = out.memptr();
+      
+      for(uword row=0; row<X_n_rows; ++row)
+        {
+        dat.copy_row(X, row);
+        
+        out_mem[row] = std::sqrt( op_var::direct_var( dat_mem, X_n_cols, norm_type) );
+        }
       }
     }
   }
