@@ -1,5 +1,5 @@
-// Copyright (C) 2012-2013 NICTA (www.nicta.com.au)
 // Copyright (C) 2012-2013 Conrad Sanderson
+// Copyright (C) 2012-2013 NICTA (www.nicta.com.au)
 // 
 // This Source Code Form is subject to the terms of the Mozilla Public
 // License, v. 2.0. If a copy of the MPL was not distributed with this
@@ -84,11 +84,11 @@ subview_elem2<eT,T1,T2>::inplace_op(const eT val)
         
         arma_debug_check( (row > m_n_rows), "Mat::elem(): index out of bounds" );
         
-             if(is_same_type<op_type, op_subview_elem_equ          >::value == true) { m_local.at(row,col)  = val; }
-        else if(is_same_type<op_type, op_subview_elem_inplace_plus >::value == true) { m_local.at(row,col) += val; }
-        else if(is_same_type<op_type, op_subview_elem_inplace_minus>::value == true) { m_local.at(row,col) -= val; }
-        else if(is_same_type<op_type, op_subview_elem_inplace_schur>::value == true) { m_local.at(row,col) *= val; }
-        else if(is_same_type<op_type, op_subview_elem_inplace_div  >::value == true) { m_local.at(row,col) /= val; }
+             if(is_same_type<op_type, op_subview_elem_equ          >::yes) { m_local.at(row,col)  = val; }
+        else if(is_same_type<op_type, op_subview_elem_inplace_plus >::yes) { m_local.at(row,col) += val; }
+        else if(is_same_type<op_type, op_subview_elem_inplace_minus>::yes) { m_local.at(row,col) -= val; }
+        else if(is_same_type<op_type, op_subview_elem_inplace_schur>::yes) { m_local.at(row,col) *= val; }
+        else if(is_same_type<op_type, op_subview_elem_inplace_div  >::yes) { m_local.at(row,col) /= val; }
         }
       }
     }
@@ -116,11 +116,11 @@ subview_elem2<eT,T1,T2>::inplace_op(const eT val)
       
       eT* colptr = m_local.colptr(col);
       
-           if(is_same_type<op_type, op_subview_elem_equ          >::value == true) { arrayops::inplace_set  (colptr, val, m_n_rows); }
-      else if(is_same_type<op_type, op_subview_elem_inplace_plus >::value == true) { arrayops::inplace_plus (colptr, val, m_n_rows); }
-      else if(is_same_type<op_type, op_subview_elem_inplace_minus>::value == true) { arrayops::inplace_minus(colptr, val, m_n_rows); }
-      else if(is_same_type<op_type, op_subview_elem_inplace_schur>::value == true) { arrayops::inplace_mul  (colptr, val, m_n_rows); }
-      else if(is_same_type<op_type, op_subview_elem_inplace_div  >::value == true) { arrayops::inplace_div  (colptr, val, m_n_rows); }
+           if(is_same_type<op_type, op_subview_elem_equ          >::yes) { arrayops::inplace_set  (colptr, val, m_n_rows); }
+      else if(is_same_type<op_type, op_subview_elem_inplace_plus >::yes) { arrayops::inplace_plus (colptr, val, m_n_rows); }
+      else if(is_same_type<op_type, op_subview_elem_inplace_minus>::yes) { arrayops::inplace_minus(colptr, val, m_n_rows); }
+      else if(is_same_type<op_type, op_subview_elem_inplace_schur>::yes) { arrayops::inplace_mul  (colptr, val, m_n_rows); }
+      else if(is_same_type<op_type, op_subview_elem_inplace_div  >::yes) { arrayops::inplace_div  (colptr, val, m_n_rows); }
       }
     }
   else
@@ -147,11 +147,11 @@ subview_elem2<eT,T1,T2>::inplace_op(const eT val)
         
         arma_debug_check( (row > m_n_rows), "Mat::elem(): index out of bounds" );
       
-             if(is_same_type<op_type, op_subview_elem_equ          >::value == true) { m_local.at(row,col)  = val; }
-        else if(is_same_type<op_type, op_subview_elem_inplace_plus >::value == true) { m_local.at(row,col) += val; }
-        else if(is_same_type<op_type, op_subview_elem_inplace_minus>::value == true) { m_local.at(row,col) -= val; }
-        else if(is_same_type<op_type, op_subview_elem_inplace_schur>::value == true) { m_local.at(row,col) *= val; }
-        else if(is_same_type<op_type, op_subview_elem_inplace_div  >::value == true) { m_local.at(row,col) /= val; }
+             if(is_same_type<op_type, op_subview_elem_equ          >::yes) { m_local.at(row,col)  = val; }
+        else if(is_same_type<op_type, op_subview_elem_inplace_plus >::yes) { m_local.at(row,col) += val; }
+        else if(is_same_type<op_type, op_subview_elem_inplace_minus>::yes) { m_local.at(row,col) -= val; }
+        else if(is_same_type<op_type, op_subview_elem_inplace_schur>::yes) { m_local.at(row,col) *= val; }
+        else if(is_same_type<op_type, op_subview_elem_inplace_div  >::yes) { m_local.at(row,col) /= val; }
         }
       }
     }
@@ -209,11 +209,11 @@ subview_elem2<eT,T1,T2>::inplace_op(const Base<eT,expr>& x)
         
         arma_debug_check( (row > m_n_rows), "Mat::elem(): index out of bounds" );
         
-             if(is_same_type<op_type, op_subview_elem_equ          >::value == true) { m_local.at(row,col)  = X.at(ri_count, ci_count); }
-        else if(is_same_type<op_type, op_subview_elem_inplace_plus >::value == true) { m_local.at(row,col) += X.at(ri_count, ci_count); }
-        else if(is_same_type<op_type, op_subview_elem_inplace_minus>::value == true) { m_local.at(row,col) -= X.at(ri_count, ci_count); }
-        else if(is_same_type<op_type, op_subview_elem_inplace_schur>::value == true) { m_local.at(row,col) *= X.at(ri_count, ci_count); }
-        else if(is_same_type<op_type, op_subview_elem_inplace_div  >::value == true) { m_local.at(row,col) /= X.at(ri_count, ci_count); }
+             if(is_same_type<op_type, op_subview_elem_equ          >::yes) { m_local.at(row,col)  = X.at(ri_count, ci_count); }
+        else if(is_same_type<op_type, op_subview_elem_inplace_plus >::yes) { m_local.at(row,col) += X.at(ri_count, ci_count); }
+        else if(is_same_type<op_type, op_subview_elem_inplace_minus>::yes) { m_local.at(row,col) -= X.at(ri_count, ci_count); }
+        else if(is_same_type<op_type, op_subview_elem_inplace_schur>::yes) { m_local.at(row,col) *= X.at(ri_count, ci_count); }
+        else if(is_same_type<op_type, op_subview_elem_inplace_div  >::yes) { m_local.at(row,col) /= X.at(ri_count, ci_count); }
         }
       }
     }
@@ -244,11 +244,11 @@ subview_elem2<eT,T1,T2>::inplace_op(const Base<eT,expr>& x)
             eT* m_colptr = m_local.colptr(col);
       const eT* X_colptr = X.colptr(ci_count);
       
-           if(is_same_type<op_type, op_subview_elem_equ          >::value == true) { arrayops::copy         (m_colptr, X_colptr, m_n_rows); }
-      else if(is_same_type<op_type, op_subview_elem_inplace_plus >::value == true) { arrayops::inplace_plus (m_colptr, X_colptr, m_n_rows); }
-      else if(is_same_type<op_type, op_subview_elem_inplace_minus>::value == true) { arrayops::inplace_minus(m_colptr, X_colptr, m_n_rows); }
-      else if(is_same_type<op_type, op_subview_elem_inplace_schur>::value == true) { arrayops::inplace_mul  (m_colptr, X_colptr, m_n_rows); }
-      else if(is_same_type<op_type, op_subview_elem_inplace_div  >::value == true) { arrayops::inplace_div  (m_colptr, X_colptr, m_n_rows); }
+           if(is_same_type<op_type, op_subview_elem_equ          >::yes) { arrayops::copy         (m_colptr, X_colptr, m_n_rows); }
+      else if(is_same_type<op_type, op_subview_elem_inplace_plus >::yes) { arrayops::inplace_plus (m_colptr, X_colptr, m_n_rows); }
+      else if(is_same_type<op_type, op_subview_elem_inplace_minus>::yes) { arrayops::inplace_minus(m_colptr, X_colptr, m_n_rows); }
+      else if(is_same_type<op_type, op_subview_elem_inplace_schur>::yes) { arrayops::inplace_mul  (m_colptr, X_colptr, m_n_rows); }
+      else if(is_same_type<op_type, op_subview_elem_inplace_div  >::yes) { arrayops::inplace_div  (m_colptr, X_colptr, m_n_rows); }
       }
     }
   else
@@ -277,11 +277,11 @@ subview_elem2<eT,T1,T2>::inplace_op(const Base<eT,expr>& x)
         
         arma_debug_check( (row > m_n_rows), "Mat::elem(): index out of bounds" );
       
-             if(is_same_type<op_type, op_subview_elem_equ          >::value == true) { m_local.at(row,col)  = X.at(ri_count, col); }
-        else if(is_same_type<op_type, op_subview_elem_inplace_plus >::value == true) { m_local.at(row,col) += X.at(ri_count, col); }
-        else if(is_same_type<op_type, op_subview_elem_inplace_minus>::value == true) { m_local.at(row,col) -= X.at(ri_count, col); }
-        else if(is_same_type<op_type, op_subview_elem_inplace_schur>::value == true) { m_local.at(row,col) *= X.at(ri_count, col); }
-        else if(is_same_type<op_type, op_subview_elem_inplace_div  >::value == true) { m_local.at(row,col) /= X.at(ri_count, col); }
+             if(is_same_type<op_type, op_subview_elem_equ          >::yes) { m_local.at(row,col)  = X.at(ri_count, col); }
+        else if(is_same_type<op_type, op_subview_elem_inplace_plus >::yes) { m_local.at(row,col) += X.at(ri_count, col); }
+        else if(is_same_type<op_type, op_subview_elem_inplace_minus>::yes) { m_local.at(row,col) -= X.at(ri_count, col); }
+        else if(is_same_type<op_type, op_subview_elem_inplace_schur>::yes) { m_local.at(row,col) *= X.at(ri_count, col); }
+        else if(is_same_type<op_type, op_subview_elem_inplace_div  >::yes) { m_local.at(row,col) /= X.at(ri_count, col); }
         }
       }
     }
